﻿Type=StaticCode
Version=3.61
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'Raspberry Pi Experiment
'Controller Code Module
'Handles GPIO controller
Sub Process_Globals
	Public AvoidDuplicates As Map
	Private Connections As Map
	'GPIO 
  	Private GPIOController As GpioController	'Define the controller
  	Private Pin1 As GpioPinDigitalOutput		'Physical Pin 12 connected to the LED 
End Sub

'Init the GPIO controler and Pins
Sub Init
	Log("Controller.Init :: Start")
	'this map is accessed from other threads so it needs to be a thread safe map
	AvoidDuplicates = Main.srvr.CreateThreadSafeMap
	Connections.Initialize
	'Init the GPIO controller and define the pins
	GPIOController.Initialize
	'Init Pin1 for the LED as Low
	Pin1.Initialize(1, False) 
	Pin1.SetShutdownOptions(True, False, "OFF")
	Log("Pin1 initialized. " & Pin1.IsInitialized )
	Log("Controller.Init :: End")
End Sub

'Handle the LED Pin state changes
Sub LEDChangeState(cnt As Int)
	Log("Controller.LEDChangeState :: Start")
	'Change the Pin state
	Pin1.State = Not(Pin1.State)
	'Update the message
	Dim State As String
	Dim StateMsg As String
	If Pin1.State = True Then
		State = "ON"
	Else
		State = "OFF"
	End If
	StateMsg = $"${DateTime.Time(DateTime.Now)} LED switched <B>${State}</B> (${cnt} times)."$
	Log(StateMsg)
	'Update the websocket with updated status
	Log("UpdateDetectorStatus for number connections: " & Connections.size)
	For Each e As Experiment In Connections.Values
		CallSubDelayed2(e, "LEDUpdateStatus", StateMsg)
	Next
	Log("Controller.LEDChangeState :: End")
End Sub

#Region HandleConnections
Public Sub NewConnection(e As Experiment, name As String)
	Log("Controller.NewConnection :: Start = Name: " & name)
	Connections.Put(name, e)
	CallSubDelayed2(e, "LEDUpdateStatus", DateTime.Time(DateTime.Now) & " Welcome to this Raspberry Pi Experiment.")
	Log("Controller.NewConnection :: End")
End Sub

Public Sub Disconnect(e As Experiment, name As String)
	Log("Controller.Disconnect :: Start = Name: " & name)
	If Connections.ContainsKey(name) = False Or Connections.Get(name) <> e Then Return
	Connections.Remove(name)
	AvoidDuplicates.Remove(name.ToLowerCase)
	'Turn the LED Off
	Pin1.State = True
	CallSubDelayed2(e, "LEDUpdateStatus", DateTime.Time(DateTime.Now) & " Thank You for using this Raspberry Pi Experiment.")
	Log("Controller.Disconnect :: End")
End Sub
#End Region
